package labss.lang.il;

import java.util.HashMap;
import java.util.Map;

import labss.lang.il.model.ILAbstractFactory;
import labss.lang.il.model.Literal;
import labss.lang.il.model.Statement;
import labss.lang.il.model.impl.ILConcreteFactory;
import labss.lang.il.visitors.EmptyIndentationStrategy;
import labss.lang.il.visitors.IndentedPrettyPrinterVisitor;
import labss.lang.il.visitors.InterpreterVisitor;
import labss.lang.il.visitors.LineIndentationStrategy;
import labss.lang.il.visitors.PrettyPrinterVisitor;
import labss.lang.il.visitors.SpaceIndentationStrategy;
import labss.lang.il.visitors.TabIndentationStrategy;
import labss.lang.il.visitors.ValidatorVisitor;

public class ProgrammaFattoriale {
	public static Statement creaProgrammaFattoriale(ILAbstractFactory f) {
		return f.createBlock(
				f.createAssignment("result", f.createInt(1)),
				f.createWhile(f.createGreaterThan(f.createVariable("n"), f.createInt(0)), f.createBlock(
						f.createAssignment("result",
								f.createMultiplication(f.createVariable("result"), f.createVariable("n"))),
						f.createAssignment("n",
								f.createSubtraction(f.createVariable("n"), f.createInt(1)))
				)));
	}
	public static Statement creaProgrammaNonValido(ILAbstractFactory f) {
		return f.createBlock(
				f.createAssignment("b1", f.createBoolean(true)),
				f.createAssignment("b2", f.createAnd(f.createVariable("b1"),f.createVariable("b2"))),
				f.createAssignment("b3", f.createGreaterThan(f.createVariable("a"), f.createInt(0))),
				f.createAssignment("b1", f.createInt(3)),
				
				f.createAssignment("result", f.createInt(1)),
				f.createWhile(f.createAddition(f.createVariable("n"), f.createBoolean(false)), f.createBlock(
						f.createAssignment("result",
								f.createMultiplication(f.createVariable("b1"), f.createVariable("n"))),
						f.createAssignment("n",
								f.createSubtraction(f.createVariable("n"), f.createInt(1)))
				)));
	}

	public static void main(String[] args) {
		Statement fact = creaProgrammaFattoriale(ILConcreteFactory.instance());

		PrettyPrinterVisitor.prettyPrint(fact, System.out);
		
		
		System.out.println();
		IndentedPrettyPrinterVisitor.prettyPrint(fact, System.out,
				new SpaceIndentationStrategy(2));
		System.out.println();
		IndentedPrettyPrinterVisitor.prettyPrint(fact, System.out,
				new TabIndentationStrategy());
		System.out.println();
		IndentedPrettyPrinterVisitor.prettyPrint(fact, System.out,
				new EmptyIndentationStrategy());
		System.out.println();
		IndentedPrettyPrinterVisitor.prettyPrint(fact, System.out, 
				new LineIndentationStrategy());

		System.out.println();
		stampa(fact);
		valida(fact);
		interpreta(fact);
		
		System.out.println("5! = " + fact(5));
		
		System.out.println();
		Statement nvp = creaProgrammaNonValido(ILConcreteFactory.instance());
		stampa(nvp);
		valida(nvp);
	}

	public static void stampa(Statement programma) {
		IndentedPrettyPrinterVisitor.prettyPrint(programma, System.out, new LineIndentationStrategy());
		System.out.println();
		System.out.flush();
	}
	public static void valida(Statement programma) {
		ValidatorVisitor.validate(programma);
	}
	public static void interpreta(Statement programma) {
		Map<String, Literal> bindings = new HashMap<String, Literal>();
		bindings.put("n", ILConcreteFactory.instance().createInt(4));
		
		InterpreterVisitor.interpret(programma, bindings);
		
		System.out.println("4! = " + bindings.get("result"));
	}
	public static int fact(int n) {
		ILAbstractFactory f = ILConcreteFactory.instance();
		Map<String, Literal> bindings = new HashMap<String, Literal>();
		bindings.put("n", f.createInt(n));
		
		InterpreterVisitor.interpret(creaProgrammaFattoriale(f), bindings);
		
		return bindings.get("result").intValue();
	}
}
