package labss.lang.il.model.impl;

import labss.lang.il.model.*;

public class ILConcreteFactory implements ILAbstractFactory {
	// Singleton
	private static ILAbstractFactory instance;
	public static ILAbstractFactory instance() {
		if (instance == null)
			instance = new ILConcreteFactory();
		return instance;
	}
	protected ILConcreteFactory() {}


	public Statement createWhile(Expression exp, Statement stm) {
		return new While(exp, stm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm) {
		return new If(exp, thenStm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm, Statement elseStm) {
		return new IfElse(exp, thenStm, elseStm);
	}
	
    public Statement createEmpty() {
    	return new Empty();
    }

    public Statement createAssignment(String var, Expression exp) {
		return new Assignment((Variable) createVariable(var), exp);
	}
	
	public Statement createBlock(Statement... stms) {
		return new Block(stms);
	}
	
	public Literal createBoolean(boolean val) {
		return Bool.instance(val);
	}
	
	public Literal createInt(int val) {
		return new Int(val);
	}
	
	public Expression createVariable(String name) {
		return new Variable(name);
	}
	
	public Expression createAddition(Expression exp1, Expression exp2) {
		return new Addition(exp1, exp2);
	}
	
	public Expression createSubtraction(Expression exp1, Expression exp2) {
		return new Subtraction(exp1, exp2);
	}
	
	public Expression createMultiplication(Expression exp1, Expression exp2) {
		return new Multiplication(exp1, exp2);
	}
	
	public Expression createDivision(Expression exp1, Expression exp2) {
		return new Division(exp1, exp2);
	}
	
	public Expression createAnd(Expression exp1, Expression exp2) {
		return new And(exp1, exp2);
	}
	
	public Expression createOr(Expression exp1, Expression exp2) {
		return new Or(exp1, exp2);
	}
	
	public Expression createNot(Expression exp) {
		return new Not(exp);
	}
	
	public Expression createEquals(Expression exp1, Expression exp2) {
		return new Equals(exp1, exp2);
	}
	
	public Expression createLessThan(Expression exp1, Expression exp2) {
		return new LessThan(exp1, exp2);
	}
	
	public Expression createGreaterThan(Expression exp1, Expression exp2) {
		return new GreaterThan(exp1, exp2);
	}
}