package labss.lang.il.visitors;

import java.io.PrintStream;

import labss.lang.il.model.*;
import labss.lang.il.model.impl.*;
import static labss.lang.il.visitors.IndentationStrategy.Kind.*;

public class IndentedPrettyPrinterVisitor implements ILVisitor {
	protected final PrintStream out;
	protected IndentationStrategy indentationStrategy;

	public static void prettyPrint(ILEntity entity, PrintStream out) {
		prettyPrint(entity, out, new SpaceIndentationStrategy());
	}
	public static void prettyPrint(ILEntity entity, PrintStream out, IndentationStrategy indentationStrategy) {
		entity.accept(new IndentedPrettyPrinterVisitor(out, indentationStrategy));
	}

	public IndentedPrettyPrinterVisitor(PrintStream out, IndentationStrategy indentationStrategy) {
		this.out = out;
		this.indentationStrategy = indentationStrategy;
	}

	public void visit(Empty exp) {
		indentationStrategy.printIndentation(out);
		out.println(";");
	}

	public void visit(While stm) {
		indentationStrategy.printIndentation(out);
		out.print("while (");
		stm.getCondition().accept(this);
		if (stm.getTrueBody() instanceof Block) {
			indentationStrategy.startIndentationPart(BLOCK);
			out.println(") {");
			stm.getTrueBody().accept(this);
			indentationStrategy.endIndentationPart();
			indentationStrategy.printIndentation(out);
			out.println("}");
		} else {
			indentationStrategy.startIndentationPart(LOOP);
			out.println(")");
			stm.getTrueBody().accept(this);
			indentationStrategy.endIndentationPart();
		}
	}

	public void visit(If stm) {
		indentationStrategy.printIndentation(out);
		out.print("if (");
		stm.getCondition().accept(this);
		if (stm.getTrueBody() instanceof Block) {
			indentationStrategy.startIndentationPart(BLOCK);
			out.println(") {");
			stm.getTrueBody().accept(this);
			out.println("}");
		} else {
			indentationStrategy.startIndentationPart(CHOICE);
			out.println(")");
			stm.getTrueBody().accept(this);
		}
		indentationStrategy.endIndentationPart();
	}
	public void visit(IfElse stm) {
		indentationStrategy.printIndentation(out);
		out.print("if (");
		stm.getCondition().accept(this);
		if (stm.getTrueBody() instanceof Block) {
			indentationStrategy.startIndentationPart(BLOCK);
			out.println(") {");
			stm.getTrueBody().accept(this);
			out.println("}");
		} else {
			indentationStrategy.startIndentationPart(CHOICE);
			out.println(")");
			stm.getTrueBody().accept(this);
		}
		indentationStrategy.endIndentationPart();
		
		indentationStrategy.printIndentation(out);
		if (stm.getFalseBody() instanceof Block) {
			indentationStrategy.startIndentationPart(BLOCK);
			out.println("else {");
			stm.getFalseBody().accept(this);
			out.println("}");
		} else {
			indentationStrategy.startIndentationPart(CHOICE);
			out.println("else");
			stm.getFalseBody().accept(this);
		}
		indentationStrategy.endIndentationPart();
	}

	public void visit(Assignment stm) {
		indentationStrategy.printIndentation(out);
		stm.getVariable().accept(this);
		out.print(" = ");
		stm.getExpression().accept(this);
		out.println(";");
	}

	public void visit(Block stm) {
		for (Statement s : stm.getStatements())
			s.accept(this);
	}

	public void visit(Bool exp) {
		out.print(exp);
	}
	public void visit(Int exp) {
		out.print(exp);
	}

	public void visit(Variable exp) {
		out.print(exp.getName());
	}

	protected void visitBinExp(BinaryExpression exp, String opStr) {
		exp.getExp1().accept(this);
		out.print(opStr);
		exp.getExp2().accept(this);
	}

	public void visit(Addition exp) {
		visitBinExp(exp, " + ");
	}
	public void visit(Subtraction exp) {
		visitBinExp(exp, " - ");
	}
	public void visit(Multiplication exp) {
		visitBinExp(exp, " * ");
	}
	public void visit(Division exp) {
		visitBinExp(exp, " / ");
	}

	public void visit(Equals exp) {
		visitBinExp(exp, " == ");
	}
	public void visit(GreaterThan exp) {
		visitBinExp(exp, " > ");
	}
	public void visit(LessThan exp) {
		visitBinExp(exp, " < ");
	}

	public void visit(And exp) {
		visitBinExp(exp, " && ");
	}
	public void visit(Or exp) {
		visitBinExp(exp, " || ");
	}
	public void visit(Not exp) {
		out.print("!");
		exp.getExp().accept(this);
	}
}
