package labss.lang.il.visitors;

import java.io.PrintStream;

import labss.lang.il.model.*;
import labss.lang.il.model.impl.*;

public class PrettyPrinterVisitor implements ILVisitor {
	protected final PrintStream out;

	public static void prettyPrint(ILEntity programma, PrintStream out) {
		programma.accept(new PrettyPrinterVisitor(out));
	}

	protected PrettyPrinterVisitor(PrintStream out) {
		this.out = out;
	}

	public void visit(Empty exp) {
		out.println(";");
	}

	public void visit(While entity) {
		out.print("while (");
		entity.getCondition().accept(this);
		out.println(") {");
		entity.getTrueBody().accept(this);
		out.println("}");
	}

	public void visit(If entity) {
		out.print("if (");
		entity.getCondition().accept(this);
		out.println(") {");
		entity.getTrueBody().accept(this);
		out.println("}");
	}
	public void visit(IfElse entity) {
		out.print("if (");
		entity.getCondition().accept(this);
		out.println(") {");
		entity.getTrueBody().accept(this);
		out.println("} else {");
		entity.getFalseBody().accept(this);
		out.println("}");
	}

	public void visit(Assignment entity) {
		entity.getVariable().accept(this);
		out.print(" = ");
		entity.getExpression().accept(this);
		out.println(";");
	}

	public void visit(Block entity) {
		for (Statement s : entity.getStatements())
			s.accept(this);
	}

	public void visit(Bool exp) {
		out.print(exp.booleanValue());
	}
	public void visit(Int exp) {
		out.print(exp.intValue());
	}

	public void visit(Variable exp) {
		out.print(exp.getName());
	}

	protected void visitBinExp(BinaryExpression exp, String opStr) {
		exp.getExp1().accept(this);
		out.print(opStr);
		exp.getExp2().accept(this);
	}

	public void visit(Addition exp) {
		visitBinExp(exp, " + ");
	}
	public void visit(Subtraction exp) {
		visitBinExp(exp, " - ");
	}
	public void visit(Multiplication exp) {
		visitBinExp(exp, " * ");
	}
	public void visit(Division exp) {
		visitBinExp(exp, " / ");
	}

	public void visit(Equals exp) {
		visitBinExp(exp, " == ");
	}
	public void visit(GreaterThan exp) {
		visitBinExp(exp, " > ");
	}
	public void visit(LessThan exp) {
		visitBinExp(exp, " < ");
	}

	public void visit(And exp) {
		visitBinExp(exp, " && ");
	}
	public void visit(Or exp) {
		visitBinExp(exp, " || ");
	}
	public void visit(Not exp) {
		out.print("!");
		exp.getExp().accept(this);
	}
}
